/*
 * Copyright (c) 2024 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef DOCA_STA_BE_H_
#define DOCA_STA_BE_H_

#include <stdint.h>

#include <doca_compat.h>
#include <doca_error.h>
#include <doca_mmap.h>

#include <doca_sta_handle.h>
#include <doca_sta_task.h>

#ifdef __cplusplus
extern "C" {
#endif

struct doca_sta;

DOCA_EXPERIMENTAL
doca_error_t doca_sta_be_create(struct doca_sta *sta, struct doca_sta_be_handle **be_handle);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_be_destroy(struct doca_sta_be_handle *be_handle);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_be_add_queue(struct doca_sta_be_handle *be_handle,
				   struct doca_mmap *sq,
				   struct doca_mmap *sq_db_reg,
				   uint16_t sq_db_offset,
				   struct doca_mmap *cq,
				   struct doca_mmap *cq_db_reg,
				   uint16_t cq_db_offset,
				   struct doca_sta_be_q_handle **be_q_handle);

/**
 * @brief This method sets the STA destroy queue tasks configuration
 *
 * @param [in] sta
 * The STA context to config
 * @param [in] task_completion_cb
 * Task completion callback
 * @param [in] task_error_cb
 * Task error callback
 *
 * @return
 * DOCA_SUCCESS - in case of success.
 * Error code - in case of failure. see doca_error_t.
 */

DOCA_EXPERIMENTAL
doca_error_t doca_sta_be_task_destroy_queue_set_conf(struct doca_sta *sta,
						     doca_sta_task_completion_cb_t task_completion_cb,
						     doca_sta_task_completion_cb_t task_error_cb);
/**
 * @brief This method allocates and initializes a STA destroy queue task
 *
 * @param [in] be_q_handle
 * The handle of the be queue that should be destroyed
 * @param [in] user_data
 * doca_data to attach to the task
 * @param [out] task
 * memcpy task to allocate
 *
 * @return
 * DOCA_SUCCESS - in case of success.
 * doca_error code - in case of failure:
 * - DOCA_ERROR_NO_MEMORY - No more tasks to allocate
 */
DOCA_EXPERIMENTAL
doca_error_t doca_sta_be_destroy_queue_task_alloc_init(struct doca_sta_be_q_handle *be_q_handle,
						       union doca_data user_data,
						       struct doca_sta_producer_task_send **task);

#ifdef __cplusplus
}
#endif

#endif /* DOCA_STA_BE_H_ */
