/*
 * Copyright (c) 2024 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef DOCA_STA_STATS_H_
#define DOCA_STA_STATS_H_

#include <stdint.h>

#include <doca_error.h>

#include <doca_sta_handle.h>

#ifdef __cplusplus
extern "C" {
#endif

struct doca_sta;

enum dpa_sta_eu_type {
	DOCA_STA_EU_TYPE_UNKNOWN = 0,
	DOCA_STA_EU_COMP,
	/**< Completion handler (eu) type */
	DOCA_STA_EU_TX,
	/**< Transmission handler (eu) type */
	DOCA_STA_EU_BEQ,
	/**< Back-end queue handler (eu) type */
	DOCA_STA_EU_MAX,
	/**< Denote all handlers type */
};

/**
 * @brief doca sta execution unit (handler) counter entry
 */
struct doca_sta_eu_ctr_entry {
	const char *name;
	/**< name of the counter */
	const uint64_t *val;
	/**< value of the counter */
};

DOCA_EXPERIMENTAL
doca_error_t doca_sta_get_eu_stats(struct doca_sta_eu_handle *eu_handle,
				   const struct doca_sta_eu_ctr_entry **entries,
				   uint16_t *num_entries);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_eu_reset_stats_handle(struct doca_sta_eu_handle *eu_handle);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_eu_reset_stats_type(struct doca_sta *sta, enum dpa_sta_eu_type eu_type);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_get_eu_connect_qp_stats(struct doca_sta_eu_handle *eu_handle,
					      struct doca_sta_qp_handle **arr,
					      uint16_t *arr_size);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_get_be_queue_stats(struct doca_sta_be_handle *be_handle,
					 struct doca_sta_be_q_handle *be_q_handle,
					 const struct doca_sta_eu_ctr_entry **entries,
					 uint16_t *num_entries);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_get_ns_stats(struct doca_sta_subs_handle *subs_handle,
				   struct doca_sta_ns_handle *ns_handle,
				   const struct doca_sta_eu_ctr_entry **entries,
				   uint16_t *num_entries);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_get_eu_handle(const struct doca_sta *sta,
				    struct doca_sta_eu_handle **eu_handle_arr,
				    uint32_t *arr_size);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_get_eu_name(const struct doca_sta_eu_handle *eu_handle, const char **name);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_get_eu_type(const struct doca_sta_eu_handle *eu_handle, enum dpa_sta_eu_type *type);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_get_eu_id(const struct doca_sta_eu_handle *eu_handle, uint16_t *eu_id);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_get_eu_port(const struct doca_sta_eu_handle *eu_handle, uint16_t *port);

#ifdef __cplusplus
}
#endif

#endif /* DOCA_STA_STATS_H_ */
