/*
 * Copyright (c) 2024 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef DOCA_STA_SUBSYSTEM_H_
#define DOCA_STA_SUBSYSTEM_H_

#include <stdint.h>

#include <doca_compat.h>
#include <doca_error.h>

#include <doca_sta_handle.h>
#include <doca_sta_task.h>

#ifdef __cplusplus
extern "C" {
#endif

struct doca_sta;
struct doca_dev;

DOCA_EXPERIMENTAL
doca_error_t doca_sta_subsystem_create(struct doca_sta *sta,
				       const char *nqn,
				       struct doca_sta_subs_handle **subs_handle);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_subsystem_add_dev(struct doca_sta_subs_handle *subs_handle, const struct doca_dev *dev);

/**
 * @brief This method sets the STA remove namespace tasks configuration
 *
 * @param [in] sta
 * The STA context to config
 * @param [in] task_completion_cb
 * Task completion callback
 * @param [in] task_error_cb
 * Task error callback
 *
 * @return
 * DOCA_SUCCESS - in case of success.
 * Error code - in case of failure. see doca_error_t.
 */

DOCA_EXPERIMENTAL
doca_error_t doca_sta_subsystem_task_rm_ns_set_conf(struct doca_sta *sta,
						    doca_sta_task_completion_cb_t task_completion_cb,
						    doca_sta_task_completion_cb_t task_error_cb);
DOCA_EXPERIMENTAL
doca_error_t doca_sta_subsystem_destroy(struct doca_sta_subs_handle *subs_handle);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_subsystem_add_ns(struct doca_sta_subs_handle *subs_handle,
				       uint32_t fe_ns_id,
				       uint32_t ns_block_size,
				       uint32_t be_ns_id,
				       struct doca_sta_be_handle *be_handle,
				       struct doca_sta_ns_handle **ns_handle);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_subsystem_rm_ns(struct doca_sta_subs_handle *subs_handle, struct doca_sta_ns_handle *ns_handle);

/**
 * @brief This method allocates and initializes a STA remove namespace task
 *
 * @param [in] subs_handle
 * The subsystem handle
 * @param [in] ns_handle
 * The handle of the namespace that should be removed
 * @param [in] user_data
 * doca_data to attach to the task
 * @param [out] task
 * memcpy task to allocate
 *
 * @return
 * DOCA_SUCCESS - in case of success.
 * doca_error code - in case of failure:
 * - DOCA_ERROR_NO_MEMORY - No more tasks to allocate
 */
DOCA_EXPERIMENTAL
doca_error_t doca_sta_subsystem_task_rm_ns_alloc_init(struct doca_sta_subs_handle *subs_handle,
						      struct doca_sta_ns_handle *ns_handle,
						      union doca_data user_data,
						      struct doca_sta_producer_task_send **task);

#ifdef __cplusplus
}
#endif

#endif /* DOCA_STA_SUBSYSTEM_H_ */
