/*
 * Copyright (c) 2024 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef DOCA_STA_TASK_H_
#define DOCA_STA_TASK_H_

#include <stdint.h>

#include <doca_error.h>
#include <doca_types.h>

#ifdef __cplusplus
extern "C" {
#endif

struct doca_task;
struct doca_sta;

struct doca_sta_producer_task_send;

/**
 * @brief Function to execute on STA task completion.
 *
 * @details This function is called by doca_pe_progress() when related task identified as completed successfully.
 * When this function called the ownership of the task object passed from DOCA back to user.
 * Inside this callback user may decide on the task object:
 * - re-submit task with doca_task_submit(); task object ownership passed to DOCA
 * - release task with doca_task_free(); task object ownership passed to DOCA
 * - keep the task object for future re-use; user keeps the ownership on the task object
 * Inside this callback the user shouldn't call doca_pe_progress().
 * Please see doca_pe_progress for details.
 *
 * Any failure/error inside this function should be handled internally or deferred;
 * due to the mode of nested in doca_pe_progress() execution this callback doesn't return error.
 *
 * NOTE: this callback type utilized for both successful & failed task completions.
 *
 * @param [in] task
 * STA disconnect task.
 * @param [in] task_user_data
 * The task user data
 */
typedef void (*doca_sta_task_completion_cb_t)(struct doca_sta_producer_task_send *task, union doca_data task_user_data);

DOCA_EXPERIMENTAL
struct doca_task *doca_sta_producer_send_task_as_task(struct doca_sta_producer_task_send *task);

#ifdef __cplusplus
}
#endif

#endif /* DOCA_STA_TASK_H_ */
